//! Authentication  reslated struct definitions

use ::ring::rand::SecureRandom;
use ::zeroize::Zeroize;

/// User identifier. 16 bytes for easy uuid conversion
#[derive(Debug, Copy, Clone)]
pub struct UserID([u8; 16]);

impl From<[u8; 16]> for UserID {
    fn from(raw: [u8; 16]) -> Self {
        UserID(raw)
    }
}

impl UserID {
    /// New random user id
    pub fn new(rand: &::ring::rand::SystemRandom) -> Self {
        let mut ret = Self([0; 16]);
        rand.fill(&mut ret.0);
        ret
    }
    /// length of the User ID in bytes
    pub const fn len() -> usize {
        16
    }
}
/// Authentication Token, basically just 32 random bytes
#[derive(Clone, Zeroize)]
#[zeroize(drop)]
pub struct Token([u8; 32]);

impl Token {
    /// length of the token in bytes
    pub const fn len() -> usize {
        32
    }
}

impl From<[u8; 32]> for Token {
    fn from(raw: [u8; 32]) -> Self {
        Token(raw)
    }
}

// Fake debug implementation to avoid leaking tokens
impl ::core::fmt::Debug for Token {
    fn fmt(
        &self,
        f: &mut core::fmt::Formatter<'_>,
    ) -> Result<(), ::std::fmt::Error> {
        ::core::fmt::Debug::fmt("[hidden token]", f)
    }
}

/// domain representation
/// Security notice: internal representation is utf8, but we will
/// further limit to a "safe" subset of utf8
// SECURITY: TODO: limit to a subset of utf8
#[derive(Debug, Clone, PartialEq)]
pub struct Domain(String);

impl TryFrom<&[u8]> for Domain {
    type Error = ();
    fn try_from(raw: &[u8]) -> Result<Self, ()> {
        let domain_string = match String::from_utf8(raw.into()) {
            Ok(domain_string) => domain_string,
            Err(_) => return Err(()),
        };
        Ok(Domain(domain_string))
    }
}

impl Domain {
    /// length of the User ID in bytes
    pub fn len(&self) -> usize {
        self.0.len()
    }
}

/// The Service ID is a UUID associated with the service.
#[derive(Debug, Copy, Clone)]
pub struct ServiceID([u8; 16]);

impl From<[u8; 16]> for ServiceID {
    fn from(raw: [u8; 16]) -> Self {
        ServiceID(raw)
    }
}

impl ServiceID {
    /// length of the User ID in bytes
    pub const fn len() -> usize {
        16
    }
}
